@@ -1,6 +1,13 @@
 import re
 
-__all__ = ['indent']
+# options for the indentation of text inside of xml or html nodes
+# solving issue https://github.com/leforestier/yattag/issues/38
+# while maintaining compatibility with older versions of yattag
+NO = False
+FIRST_LINE = True
+EACH_LINE = 2
+
+__all__ = ['indent', 'NO', 'FIRST_LINE', 'EACH_LINE']
 
 class TokenMeta(type):
 
@@ -26,6 +33,15 @@ class Token(TokenBase):
         
 class Text(Token):
     regex = '[^<>]+'
+    def __init__(self, *args, **kwargs):
+        super(Text, self).__init__(*args, **kwargs)
+        self._isblank = None
+        
+    @property
+    def isblank(self):
+        if self._isblank is None:
+            self._isblank = not self.content.strip()
+        return self._isblank
     
 class Comment(Token):
     regex = r'<!--((?!-->).)*.?-->'
@@ -49,7 +65,7 @@ _open_tag_start = r'''
             )?  # the attribute value is optional (we're forgiving)
         )*
     \s*'''
-    
+
 class Script(Token):
     _end_script = r'<\s*/\s*script\s*>'
     
@@ -70,6 +86,12 @@ class Style(Token):
         end_style = _end_style
     )
 
+class XMLDeclaration(Token):
+    regex = _open_tag_start.format(
+        tag_name_key = 'xmldecl_ignore',
+        tag_name_rgx = r'\?\s*xml'
+    ) + r'\?\s*>'
+
 class NamedTagTokenMeta(TokenMeta):
     def __new__(cls, name, bases, attrs):
         kls = TokenMeta.__new__(cls, name, bases, attrs)
@@ -139,7 +161,7 @@ class Tokenizer(object):
         return result
         
 tokenize = Tokenizer(
-    (Text, Comment, CData, Doctype, Script, Style, OpenTag, SelfTag, CloseTag)
+    (Text, Comment, CData, Doctype, XMLDeclaration, Script, Style, OpenTag, SelfTag, CloseTag)
 ).tokenize
 
 class TagMatcher(object):
@@ -154,20 +176,39 @@ class TagMatcher(object):
                 open_tag = self.unmatched_open.pop()
                 self.matched[open_tag] = i
                 self.matched[i] = open_tag
+                return open_tag
+            else:
+                return None
                 
         def sigopen(self, i):
             self.unmatched_open.append(i)
-                
-    def __init__(self, token_list):
+                   
+    def __init__(self, token_list, blank_is_text = False):
         self.token_list = token_list
         self.name_matchers = {}
+        self.direct_text_parents = set()
+        
         for i in range(len(token_list)):
             token = token_list[i]
-            if isinstance(token, OpenTag):
+            tpe = type(token)
+            if tpe is OpenTag:
                 self._get_name_matcher(token.tag_name).sigopen(i)
-            elif isinstance(token, CloseTag):
+            elif tpe is CloseTag:
                 self._get_name_matcher(token.tag_name).sigclose(i)
-                
+        
+        # TODO move this somewhere else
+        current_nodes = []
+        for i in range(len(token_list)):
+            token = token_list[i]
+            tpe = type(token)
+            if tpe is OpenTag and self.ismatched(i):
+                current_nodes.append(i)
+            elif tpe is CloseTag and self.ismatched(i):
+                current_nodes.pop()
+            elif tpe is Text and (blank_is_text or not token.isblank):
+                if current_nodes:
+                    self.direct_text_parents.add(current_nodes[-1])
+                        
     def _get_name_matcher(self, tag_name):
         try:
             return self.name_matchers[tag_name]
@@ -177,56 +218,139 @@ class TagMatcher(object):
             
     def ismatched(self, i):
         return i in self.name_matchers[self.token_list[i].tag_name].matched
+        
+    def directly_contains_text(self, i):
+        return i in self.direct_text_parents
                 
+new_line_rgx= re.compile(r'(\r?\n)', flags = re.MULTILINE)
             
-def indent(string, indentation = '  ', newline = '\n', preserve_blank_text = False):
+def indent(string, indentation = '  ', newline = '\n', indent_text = NO, blank_is_text = False):
+    """
+    takes a string representing a html or xml document and returns
+     a well indented version of it
+    
+    arguments:
+    - string: the string to process
+    - indentation: the indentation unit (default to two spaces)
+    - newline: the string to be use for new lines
+      (default to  '\\n', could be set to '\\r\\n' for example)
+    - indent_text:
+        the value of this option should one of yattag.NO, yattag.FIRST_LINE or yattag.EACH_LINE
+        
+        if indent_text is NO, text nodes won't be indented, and the content
+         of any node directly containing text will be unchanged:
+         
+            <p>Hello</p> will be unchanged
+            
+            <p><strong>Hello</strong> world!</p> will be unchanged
+             since ' world!' is directly contained in the <p> node.
+            
+            This is the default since that's generally what you want for HTML.
+        
+        if indent_text is FIRST_LINE, the first line of text nodes will be indented:
+        
+            <p>Hello</p>
+            
+            would result in
+            
+            <p>
+              hello
+            </p>
+            
+            and:
+            
+            <p>Hello,
+            where are the keys?</p>
+            
+            would result in
+            
+            <p>
+              hello,
+            where are the keys?
+            </p>
+            
+        if indent_text is EACH_LINE, each line inside the text nodes will be indented:
+        
+            <code class="scala-source">
+            object HelloWorld {
+                def main(args: Array[String]) {
+                    println("Hello, world!")
+                }
+            }
+            </code>
+            
+            would result in
+            
+            <code class="scala-source">
+  
+              object HelloWorld {
+                  def main(args: Array[String]) {
+                      println("Hello, world!")
+                  }
+              }
+              
+            </code>
+        
+    - blank_is_text:
+        if False, completely blank texts are ignored. That is the default.
+    """ 
     tokens = tokenize(string)
-    ismatched = TagMatcher(tokens).ismatched
+    tag_matcher = TagMatcher(tokens, blank_is_text = blank_is_text)
+    ismatched = tag_matcher.ismatched
+    directly_contains_text = tag_matcher.directly_contains_text
     result = []
     append = result.append
     level = 0
     sameline = 0
     was_just_opened = False
+    tag_appeared = False
     def _indent():
+        if tag_appeared:
+            append(newline)
         for i in range(level):
             append(indentation)
-    for i in range(len(tokens)):
-        token = tokens[i]
+    def _append_text(text):
+        if not sameline:
+            _indent()
+        if indent_text is EACH_LINE:
+            append(new_line_rgx.sub(r'\1' + indentation * level, text))
+        else:
+            append(text)       
+    for i,token in enumerate(tokens):
         tpe = type(token)
         if tpe is Text:
-            if preserve_blank_text or token.content.strip():
-                append(token.content)
-                sameline = sameline or 1
+            if blank_is_text or not token.isblank:
+                _append_text(token.content)
                 was_just_opened = False
         elif tpe is OpenTag and ismatched(i):
+            was_just_opened = True
             if sameline:
                 sameline += 1
             else:
-                i == 0 or append(newline)
                 _indent()
-                was_just_opened = True
+            if indent_text is NO and directly_contains_text(i):
+                sameline = sameline or 1
             append(token.content)
             level += 1
+            tag_appeared = True
         elif tpe is CloseTag and ismatched(i):
             level -= 1
+            tag_appeared = True
             if sameline:
                 sameline -= 1
-            elif was_just_opened:
-                was_just_opened = False
-            else:
-                append(newline)
-                _indent()
-            append(token.content)
-        else:
-            if not sameline:
-                i == 0 or append(newline)
+            elif not was_just_opened:
                 _indent()
             append(token.content)
             was_just_opened = False
+        else:
+            if not sameline:
+                _indent()
+            append(token.content)
+            was_just_opened = False
+            tag_appeared = True
     return ''.join(result)
     
 if __name__ == '__main__':
     import sys
-    inp = sys.stdin.read()
-    print(indent(inp))
+    print(indent(sys.stdin.read()))
 
