@@ -1,5 +1,8 @@
 __all__ = ['SimpleDoc']
 
+class DocError(Exception):
+    pass
+
 class SimpleDoc(object):
 
     """
@@ -41,13 +44,23 @@ class SimpleDoc(object):
                 self.doc.current_tag = self.parent_tag
          
     class DocumentRoot(object):
+    
+        class DocumentRootError(DocError, AttributeError):
+            # Raising an AttributeError on __getattr__ instead of just a DocError makes it compatible
+            # with the pickle module (some users asked for pickling of SimpleDoc instances).
+            # I also keep the DocError from earlier versions to avoid possible compatibility issues
+            # with existing code.
+            pass
+    
         def __getattr__(self, item):
-            raise DocError("DocumentRoot here. You can't access anything here.")
+            raise SimpleDoc.DocumentRoot.DocumentRootError("DocumentRoot here. You can't access anything here.")
 
-    def __init__(self):
+    def __init__(self, stag_end = ' />'):
         self.result = []
         self.current_tag = self.__class__.DocumentRoot()
         self._append = self.result.append
+        assert stag_end in (' />', '/>', '>') 
+        self._stag_end = stag_end
         
     def tag(self, tag_name, *args, **kwargs):
         """
@@ -202,15 +215,27 @@ class SimpleDoc(object):
         Example::
         
             doc.stag('img', src = '/salmon-plays-piano.jpg')
-            # appends <img src="/salmon-plays-piano.jpg /> to the document
+            # appends <img src="/salmon-plays-piano.jpg" /> to the document
+        
+        If you want to produce self closing tags without the ending slash (HTML5 style),
+        use the stag_end parameter of the SimpleDoc constructor at the creation of the
+        SimpleDoc instance.
+        
+        Example::
+            
+            >>> doc = SimpleDoc(stag_end = '>')
+            >>> doc.stag('br')
+            >>> doc.getvalue()
+            '<br>'
         """
         if args or kwargs:
-            self._append("<%s %s />" % (
+            self._append("<%s %s%s" % (
                 tag_name,
                 dict_to_attrs(_attributes(args, kwargs)),
+                self._stag_end
             ))
         else:
-            self._append("<%s />" % tag_name)
+            self._append("<%s%s" % (tag_name, self._stag_end))
             
     def cdata(self, strg, safe = False):
         """
@@ -332,9 +357,6 @@ class SimpleDoc(object):
                 del self.current_tag.attrs['class']
             except KeyError:
                 pass
-
-class DocError(Exception):
-    pass
           
 def html_escape(s):
     if isinstance(s,(int,float)):
